/*
  setupServos_MEGA
  Setup the min/max positions of HerkuleX servos thanks to the HerkuleXLib
  (see www.benoitpuel.com/blog/telechargements/) using an Arduino MEGA 
  board.

  The serial terminal is used to printout messages.

  This example code is in the public domain.

  modified 2 March 2016
  by Benoit Puel
 */

#include <HkxSetup.h>

// global variables
byte ID = 0; // ID of the servo
HkxPrint* printoutLib;
HkxPrint* printoutCode;
HkxCommunication* communication;
HkxSetup* setupServos;
String message;
uint16_t overloadPWMThreshold;
int16_t minPosition, maxPosition;

void setup() {
  delay(2000); // wait 2 seconds to open the serial monitor

  printoutLib = new HkxPrint(Serial, 9600);  // Printout errors on the serial monitor
  printoutCode = new HkxPrint(*printoutLib, true, true, true);  // Printout errors on the serial monitor
  communication = new HkxCommunication(HKX_115200, Serial1, *printoutLib);  // Communication with the servo on Serial1
  setupServos = new HkxSetup(*communication, *printoutLib);  // setup the servos

  setupServos->setAllBaudRate(HKX_666666);
  setupServos->clearOneStatus(ID);
  setupServos->setAllAcceleration (50, 504); //Default values

  setupServos->getPWMPositionLimits(ID, &overloadPWMThreshold, &minPosition, &maxPosition);
  
  // All servos torque break and led off, but active servo torque free and led blue
  setupServos->setAllTorqueLEDControl(HKX_TORQUE_BREAK, HKX_LED_OFF);
  setupServos->setOneTorqueLEDControl(ID, HKX_TORQUE_FREE, HKX_LED_BLUE);
  printoutCode->infoPrint(" \" ID \" then number of the servo");
  printoutCode->infoPrint(" \" min \" to set the current position as the mimum");
  printoutCode->infoPrint(" \" max \" to set the current position as the maxium");

}

void loop() {
  int16_t currentPosition;

  // Get the behaviour
  byte error = setupServos->getBehaviour(ID, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, &currentPosition, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE);
  if(error == 0){
    message = "ID=";
    message += ID;
    message += " position=";
    message += currentPosition;
    printoutCode->infoPrint(message);
  } else if(error == 1){
    printoutCode->errorPrint("Input not correct");
  } else if(error == 2){
    printoutCode->errorPrint("Servo not connected");
  } else if(error == 3){
    printoutCode->errorPrint("Data not consistent");
  }

  // Get 
  if (Serial.available() > 0) {
    String income = Serial.readString();
    if(income.equals("ID")){
      printoutCode->infoPrint("Enter the ID");
      while(Serial.available() < 1) {};
      setupServos->setOneTorqueLEDControl(ID, HKX_TORQUE_BREAK, HKX_LED_OFF);
      String income = Serial.readString();
      ID = income.toInt();
      setupServos->setOneTorqueLEDControl(ID, HKX_TORQUE_FREE, HKX_LED_BLUE);
      setupServos->getPWMPositionLimits(ID, &overloadPWMThreshold, &minPosition, &maxPosition);
      printoutCode->infoPrint("ID has been modified");
    } else if(income.equals("min")){
      minPosition = currentPosition;
      setupServos->setOnePWMPositionLimits(ID, overloadPWMThreshold, minPosition, maxPosition);
      message = "min position of ID=";
      message += ID;
      message += " has been modified";
      printoutCode->infoPrint(message);
    } else if(income.equals("max")){
      maxPosition = currentPosition;
      setupServos->setOnePWMPositionLimits(ID, overloadPWMThreshold, minPosition, maxPosition);
      message = "max position of ID=";
      message += ID;
      message += " has been modified";
      printoutCode->infoPrint(message);
    } else {
      printoutCode->warningPrint("Incorrect input");
    }
    

  }
  
  delay(1000); // refresh every second
}
